#!/bin/sh

echo "Generating HTML doc index files..."

cd "`dirname "$0"`/../include"
includes="platform.sh global.sh"
for f in $includes ; do
    [ -f "./$f" ] && . ./$f
done

if [ `locale charmap` != 'UTF-8' ]; then
    # workaround GNU grep 2.23 and 2.24 bug
    utf8_locales="`locale -a | "$GREP" -Ei '\.utf-?8$'`"
    for locale_regex in '^C\.UTF-?8$' '^en_US\.UTF-?8$' '^en_[A-Z]{2}\.UTF-?8$' '^en_.*\.UTF-?8$' ''; do
        acceptable_locale=`echo "$utf8_locales" | "$GREP" -Ei "$locale_regex" | head -n1`
        if [ -n "$acceptable_locale" ]; then
            LC_CTYPE=$acceptable_locale
            export LC_CTYPE
            if [ -n "$LC_ALL" ]; then
                LANG="$LC_ALL"
                export LANG
                unset LC_ALL
            fi
            break
        fi
    done
fi

indexwrt() {
    echo "$1" >> $indexfile_tmp
}

gen_html_index() {
    htmldoc="$PREFIX/share/doc/html"
    FSEP=af8oe45ut8gge045mdo9r8
    langs="`ls -d \"$htmldoc\"/?? 2>/dev/null || true`"
    for lang in $langs; do
        [ -d "$lang" ] || continue

        lang="`basename $lang`"

        indexfile="$htmldoc/$lang/index.html"
        indexfile_tmp=`mktemp /tmp/drweb.indexfile_tmp.XXXXXXXXXX`

        if [ "$lang" = ru ]; then
            title='Справка Dr.Web'
        else
            title='Dr.Web Help'
        fi

        indexwrt '<html>'
        indexwrt '<head>'
        indexwrt "<title>$title</title>"
        indexwrt '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />'
        indexwrt '<link rel="stylesheet" type="text/css" href="../toc_style.css"/>'
        indexwrt '</head>'
        indexwrt '<body>'
        indexwrt "<h1 class=\"toc_title\">$title</h1>"
        indexwrt '<div class="toc_splitter"></div>'

        indexwrt '<ul>'

        headfile=`mktemp /tmp/drweb.headfile.XXXXXXXXXX`
        for f in `find "$htmldoc/$lang" -name '*content.htm'`; do
            grepfile=`mktemp /tmp/drweb.grepfile.XXXXXXXXXX`
            grep 'class=\"heading.\"' "$f" > $grepfile

            while read line; do
                text="`echo $line | sed 's|.*<span .*>\(.*\)</span>.*|\1|g'`"
                class="`echo $line | sed 's|.*class=\"\([a-z0-9]*\)\".*|\1|g'`"
                path="`dirname $f`/`echo $line | sed 's|.*href=\"\([^\"]*\)\".*|\1|g'`"
                href="`echo $path | sed 's|.*/\([^/]*/[^/]*\)$|\1|'`"
                id="`grep name=\\"id\\" $path | sed 's|.*name=\"id\" content=\"\([0-9]*\)[\",].*|\1|'`"
                id=`printf %05d $id`

                echo "$id$FSEP$text$FSEP$class$FSEP$href" >> $headfile
            done < $grepfile
            sort $headfile > $grepfile
            mv $grepfile $headfile
        done

        nonempty=`mktemp /tmp/drweb.nonempty.XXXXXXXXXX`
        sect=

        while read line; do
            text="`echo $line | "$AWK" -F $FSEP '{print $2}'`"
            class="`echo $line | "$AWK" -F $FSEP '{print $3}'`"
            href="`echo $line | "$AWK" -F $FSEP '{print $4}'`"

            if [ "$class" = heading1 ]; then
                if [ -n "$sect" ]; then
                    indexwrt '    </ul>'
                    indexwrt '  </li>'
                fi

                indexwrt "  <li class=\"$class\">"
                indexwrt "    <a href=\"$href\" target=\"_top\">$text</a>"
                indexwrt '    <ul>'
                sect=1
            elif [ "$class" = heading2 ]; then
                indexwrt "      <li class=\"$class\">"
                indexwrt "        <a href=\"$href\" target=\"_top\">$text</a>"
                indexwrt '      </li>'
            fi

            echo 1 > $nonempty
        done < $headfile

        rm $headfile

        [ -n "$sect" ] && indexwrt '    </ul>'

        indexwrt '</ul>'

        indexwrt '<div class="toc_splitter"></div>'
        if [ "$lang" = ru ]; then
            indexwrt "<div class=\"\">&copy; &laquo;Доктор Веб&raquo;, 2017.</div>"
        else
            indexwrt "<div class=\"\">&copy; Doctor Web, 2017.</div>"
        fi

        indexwrt '</body>'

        indexwrt '</html>'

        if [ -s "$nonempty" ]; then
            mv $indexfile_tmp $indexfile
            chmod 644 $indexfile
        else
            rm -rf "`dirname $indexfile`"
        fi

        rm -f $indexfile_tmp $nonempty

    done
}

gen_html_index
