checkRoot() {
    [ -n "$DONT_CHECK_ROOT" ] && return

if [ "`$ID -u`" != "0" ]; then
  echo
  eval_gettext "This script must be executed with root privileges." ; echo
  echo
  exit 1
fi
}

envInit(){
if [ -f /usr/xpg4/bin/awk ]; then
    AWK=/usr/xpg4/bin/awk
else
    AWK=`which awk 2>/dev/null`
    [ -x "$AWK" ] || AWK=awk
fi

if [ -f /usr/xpg4/bin/sed ]; then
    SED=/usr/xpg4/bin/sed
else
    SED=`which sed 2>/dev/null`
    [ -x "$SED" ] || SED=sed
fi

if [ -f /usr/xpg4/bin/grep ]; then
    GREP=/usr/xpg4/bin/grep
else
    GREP=`which grep 2>/dev/null`
    [ -x "$GREP" ] || GREP=grep
fi

if [ -f /usr/xpg4/bin/id ]; then
    ID=/usr/xpg4/bin/id
else
    ID=`which id 2>/dev/null`
    [ -x "$ID" ] || ID=id
fi

if [ -f /usr/xpg4/bin/tr ]; then
    TR=/usr/xpg4/bin/tr
else
    TR=`which tr`
    [ -x "$TR" ] || TR=tr
fi

if type gpg2 >/dev/null 2>&1; then
    GPG=gpg2
elif type gpg >/dev/null 2>&1; then
    GPG=gpg
fi
}

# Not gettexted becose not used now
BackupFile()
{
    if [ -f "$1" ] ; then
        if [ -f "$1.drwebsave" ] ; then
            CONF_DRWEBSAVE="yes"
            TIMESTAMP=`date "+%Y%m%d-%k_%M_%S"`
            echo "Info: $1.drwebsave exists. Saving $1 as $1.$TIMESTAMP"
            cp -p "$1" "$1.$TIMESTAMP"
            if [ $? -ne 0 ] ; then
                echo "Error: can't backup $1!"
                return 3
            fi
        fi
        if [ "$CONF_DRWEBSAVE" != "yes" ] ; then
            cp -p "$1" "$1.drwebsave"
            if [ $? -ne 0 ] ; then
                echo "Error: can't backup $1!"
                return 2
            else
                echo "Success: backup file $1 is saved to $1.drwebsave."
            fi
        fi
    else
        echo ""
        echo "Error: can't find file $1 for backup!"
        echo ""
        return 1
    fi
    return 0;
}

showyesno() {
if [ "$DRWEB_NON_INTERACTIVE" ] ; then
    return 0
fi
printf_str="$1"
shift
retval=0
while true ; do
    printf "$printf_str (YES/no) " "$*" 2>/dev/null
    read yesno
    case "$yesno" in
        y | yes | Y | Yes | YES | "")
	    retval=0
	    echo "yes"
            break
        ;;
        n | no | N | No | NO)
	    retval=1
	    echo "no"
	    break
        ;;
        *)
            eval_gettext "Please enter yes or no." ; echo
        ;;
    esac
done
return $retval
}

shownoyes() {
if [ "$DRWEB_NON_INTERACTIVE" ] ; then
    return 0
fi
printf_str="$1"
shift
retval=0
while true ; do
    printf "$printf_str (yes/NO) " "$*" 2>/dev/null
    read yesno
    case "$yesno" in
        y | yes | Y | Yes | YES)
	    retval=0
	    echo "yes"
            break
        ;;
        n | no | N | No | NO | "")
	    retval=1
	    echo "no"
	    break
        ;;
        *)
            eval_gettext "Please enter yes or no." ; echo
        ;;
    esac
done
return $retval
}

# $1 - Message type ("info", "warning" or "error")
# $2 - Text (printf string)
show_message() {
    # Process args.
    type=$1
    printf_str=$2
    shift
    shift
    message="`printf \"$printf_str\" \"$@\" 2>/dev/null`"

    # Form window title.
    StringInList 'info warning error' $type || type=info
    case "$type" in
        info) title="`eval_gettext \"Info\"`" ;;
        warning) title="`eval_gettext \"Warning\"`" ;;
        error) title="`eval_gettext \"Error\"`" ;;
    esac

    echo
    echo  '*'
    echo  "* $title: $message" | sed 's/<[^>]\+>//g'
    echo  '*'
}

# $1 - Message type ("info", "warning" or "error")
# $2 - Text (printf string)
gui_message() {
    # Process args.
    type=$1
    printf_str=$2
    shift
    shift
    message="`printf \"$printf_str\" \"$@\" 2>/dev/null`"

    # Form window title.
    StringInList 'info warning error' $type || type=info
    case "$type" in
        info) title="`eval_gettext \"Info\"`" ;;
        warning) title="`eval_gettext \"Warning\"`" ;;
        error) title="`eval_gettext \"Error\"`" ;;
    esac

    # Find some GUI dialog program and form its parameters.
    if [ -x "`which zenity 2>/dev/null`" ]; then
        dialog_program=zenity
        type_arg="--$type"
        message_arg="--text '$message' --no-wrap"
    elif [ -x "`which kdialog 2>/dev/null`" ]; then
        dialog_program=kdialog
        case "$type" in
            info) type_arg='--msgbox' ;;
            warning) type_arg='--sorry' ;;
            error) type_arg='--error' ;;
        esac
        message_arg="\"$message\""
    else
        dialog_program=
    fi
    title_arg="--title $title"

    # Show the message.
    show_message "$type" "$printf_str" "$@"
    if [ -n "$DISPLAY" ] && [ -n "$dialog_program" ]; then
        eval "$dialog_program" $title_arg $type_arg $message_arg 2>/dev/null
    fi
}

# $1 - Question text
# $2 - Title text
# Returns: 0=yes 1=no 2=no dialog program.
gui_ask() {
    # Process args.
    question_str="$1"
    title_str="$2"

    # Find some GUI dialog program and form its parameters.
    if [ -x "`which zenity 2>/dev/null`" ]; then
        dialog_program=zenity
        type_arg="--question"
        question_arg="--text \"$question_str\""
    elif [ -x "`which kdialog 2>/dev/null`" ]; then
        dialog_program=kdialog
        type_arg="--yesno"
        question_arg="\"$question_str\""
    else
        dialog_program=
        return 2
    fi
    title_arg="--title '$title_str'"

    # Show the message.
    if [ -n "$dialog_program" ]; then
        eval "$dialog_program $title_arg $type_arg $question_arg"
    else
        false
    fi

    return $?
}

checkNum() {
    echo "$1" | "$GREP" -q '^[0-9][0-9]*$'
    return $?
}

MakeTmpFile() {
    sc_name=`basename $0`
    tm_file=`mktemp /tmp/$sc_name.XXXXXXXXXX`
    chmod 644 $tm_file
    echo $tm_file
}

# Returns 0 if there is specified key in specified section
# of ini-type config file. Returns 1 otherwise.
#
# $1 - config file
# $2 - section name regexp
# $3 - key name regexp
IniHaveKey() {
    perl_script="
        \$sect='$2';
        \$key='$3';
        open FILE, \"$1\";
        while (<FILE>) {
            next unless /^\\s*\[\$sect\]/;
            while (<FILE>) {
                last if /^\\s*\[/;
                exit 0 if /^\\s*\$key\\s*=/;
            }
        }
        exit 1;"
    perl -e "$perl_script"
}

# Prints list of values of some key of specified section
# of ini-type config file.
#
# $1 - config file
# $2 - section name regexp
# $3 - key name regexp
IniKeyValues() {
    perl_script="
        \$sect='$2';
        \$key='$3';
        open FILE, \"$1\";
        while (<FILE>) {
            next unless /^\\s*\[\$sect\]/;
            while (<FILE>) {
                last if /^\\s*\[/;
                (\$val)=(/^\\s*\$key\\s*=\\s*(.*)/);
                next if !\$val;
                \$val=~s/\\s*,\\s*/ /g;
                \$keys.=\" \$val\";
            }
        }
        \$keys=~s/^\\s*(.*)\\s*\$/\$1/;
        print \"\$keys\\n\";"
    perl -e "$perl_script"
}

# Executes some Perl code for each line of specified section
# of ini-type config file.
#
# $1 - config file
# $2 - section name regexp
# $3 - perl code
IniSectProc() {
    TR_SP_SCRIPT="
        \$sect='$2';
        open FILE, \$ARGV[0];
        while (<FILE>) {
            print; next unless /^\\s*\[\$sect\]/;
            while (<FILE>) {
                if (/^\\s*\[/) { print; last }
                $3 print;
            }
        }"
    tmpfile=`MakeTmpFile`
    perl -e "$TR_SP_SCRIPT" $1 > $tmpfile || return 1
    mv -f $tmpfile $1
}

# Not gettexted becose not used now
AskUpdateConfig() {
    old_config_file=$1
    new_config_file=$2
    dont_ask=$3

    if diff $old_config_file $new_config_file >/dev/null; then
        echo "$old_config_file is up-to-date, it is not necessary to modify it."
        rm -f $new_config_file
    else
        if [ -z "$dont_ask" ]; then
            echo
            echo "Your $old_config_file is about to be modified."
            echo "Backup of the original would be kept."
            echo "Here's the diff:"
            echo
            diff -u $old_config_file $new_config_file && true
            echo
            showyesno 'Do you agree with these changes?'
        else
            true
        fi
        if [ "$?" = "0" ]; then
            BackupFile $old_config_file
            mv -f $new_config_file $old_config_file
            echo "Your $old_config_file has been altered by this script."
            echo "The original has been backed up."
        else
            rm -f $new_config_file
        fi
    fi
}

# GNU/Linux have seq. FreeBSD have jot instead of seq.
# Solaris have neither seq nor jot. Using Perl as always...
#
# Usage: Seq <from> <to>
Seq() {
    perl -e "\$,=\" \"; print $1..$2;"
}

# Checks if string is in the list.
#
# $1 - list
# $2 - string
StringInList() {
    if  (echo $1 | grep "^$2[[:space:]]" >/dev/null) ||
        (echo $1 | grep "[[:space:]]$2\$" >/dev/null) ||
        (echo $1 | grep "^$2\$" >/dev/null) ||
        (echo $1 | grep "[[:space:]]$2[[:space:]]" >/dev/null); then
        return 0
    else
        return 1
    fi
}

WhichPerl() {
    if [ -x "`which perl 2>/dev/null`" ] ; then
        return 0
    else
        return 1
    fi
}

run_in_terminal() {
    if [ -z "$DISPLAY" -o \( -t 0 -a -t 1 \) ] ; then
        "$@"
        return $?
    fi
    if [ -z "$XDG_CURRENT_DESKTOP" ] ; then
        if [ -n "$GNOME_DESKTOP_SESSION_ID" -o "$DESKTOP_SESSION" = gnome ] ; then
            XDG_CURRENT_DESKTOP=GNOME
        elif [ -n "$MATE_DESKTOP_SESSION_ID" -o "$DESKTOP_SESSION" = mate ] ; then
            XDG_CURRENT_DESKTOP=MATE
        elif [ -n "$DESKTOP_SESSION" ] ; then
            XDG_CURRENT_DESKTOP="$DESKTOP_SESSION"
        fi
    fi
    if [ -n "$XDG_CURRENT_DESKTOP" ] ; then
        OIFS="$IFS"
        IFS=:
        for desktop in $XDG_CURRENT_DESKTOP; do
            IFS="$OIFS"
            case "$desktop" in
            GNOME|Unity|X-Cinnamon)
                if [ -x "`which gnome-terminal 2>/dev/null`" ] ; then
                    waitfile="`mktemp drwinst.wait.XXXX`"
                    gnome-terminal -x sh -c "trap 'rm -f \"$waitfile\"' EXIT HUP INT PIPE TERM; \"\$@\"" '' "$@"
                    while [ -f "$waitfile" ]; do
                        sleep 1
                    done
                    return 0
                fi
                ;;
            MATE)
                if [ -x "`which mate-terminal 2>/dev/null`" ] ; then
                    mate-terminal --disable-factory -x "$@"
                    return $?
                fi
                ;;
            KDE)
                if [ -x "`which konsole 2>/dev/null`" ] ; then
                    konsole --nofork -e "$@"
                    return $?
                fi
                ;;
            XFCE)
                if [ -x "`which xfce4-terminal 2>/dev/null`" ] ; then
                    xfce4-terminal --disable-server -x "$@"
                    return $?
                fi
                ;;
            LXQt)
                if [ -x "`which qterminal 2>/dev/null`" ] ; then
                    qterminal -e "$@"
                    return $?
                fi
                ;;
            LXDE)
                if [ -x "`which lxterminal 2>/dev/null`" ] ; then
                    lxterminal -e "$@"
                    return $?
                fi
                ;;
            fly)
                if [ -x "`which fly-term 2>/dev/null`" ] ; then
                    fly-term -e "$@"
                    return $?
                fi
                ;;
            esac
        done
    fi
    if [ -x "`which urxvt 2>/dev/null`" ] ; then
        urxvt -e "$@"
    elif [ -x "`which uxterm 2>/dev/null`" ] ; then
        uxterm -e "$@"
    elif [ -x "`which xterm 2>/dev/null`" ] ; then
        xterm -e "$@"
    elif [ -x "`which x-terminal-emulator 2>/dev/null`" ] ; then
        x-terminal-emulator -e "$@"
    else
        "$@"
    fi
    return $?
}

envInit
checkRoot
